#!/usr/bin/env python

from daemon import Daemon
from time import sleep
import subprocess
import requests
import urllib2
import json
import os

def Task():
    def __init__(self, token):
        self.token = token
    return True
def Config():
    return True

def start(config):
    interval = config.interval
    tasks = Task(config.token)

    while True:
        # update from server
        tasks.get_updates() # check whether have new updates

        # run tasks
        # run all task (including old tasks) & save result
        tasks.run_and_save_result()

        # send result to center
        if tasks.send_back_result() is False:
            times = 1;
            while (times < 10):
                sleep(10 + 2 * 10)
                if tasks.send_again():
                    break

        # set interval for next run
        interval = tasks.get_interval()
        config.set_interval(interval)
        sleep(interval)

if __name__ == '__main__':
    # run empire as daemon
    class Empire(Daemon):
        def run(self):
            start(config)

    config = Config('/path/to/config')
    pid_file = config.pid_file
    empire = Empire(pid_file)
    if os.path.isfile(pid_file):
        print 'Exit...'
        empire.stop()
    else:
        print 'Start...'
        empire.start()



# -*- coding: utf-8 -*-

from time import sleep
from .daemon import Daemon
from .config import Config
from .task import Task

import subprocess
import requests
import urllib2
import json

def launch(config):
    interval = config.interval
    tasks = Task(config)

    while True:
        # update from server
        tasks.get_updates() # check whether have new updates

        # run tasks
        # run all task (including old tasks) & save result
        tasks.handle_tasks()

        # send result to center
        tasks.send_back_result()

        # set interval for next run
        interval = tasks.get_interval()
        tasks.set_interval(interval)
        tasks.sleep()

class EmpireAgent(Daemon):
    def run(self, config):
        launch(config)



# -*- coding: utf-8 -*-

import json

class Config(object):
    def __init__(self, config_path):
        with open(config_path) as file_content:
             config_json = json.load(file_content)
        self.interval = config_json.get('interval', 300) # interval default: 5 mins
        self.pid_file = config_json.get('pid_file', '/tmp/empire-agent.pid')
        self.server = config_json.get('server', None)
        self.cache_file = config_json.get('cache', '/tmp/empire-agent.cache')



import logging, os

LOG_FORMAT = '%(asctime)s %(levelname)s %(message)s'
DATE_FORMAT = '%Y-%d-%m %H:%M:%S'
LOG_PATH = '/var/log/empire-agent/'

def log_init(log_format = LOG_FORMAT,
        log_path = LOG_PATH, date_format = DATE_FORMAT):
    if not os.path.exists(log_path):
        os.makedirs(log_path)

    logging.basicConfig(filename = log_path + 'empire-agent.log',
                       format = log_format,
                       datefmt = date_format,
                       level = logging.DEBUG)
    logger = logging.getLogger()
    global log
    log = logger



# -*- coding: utf-8 -*-

from time import sleep
from time import time
from multiprocessing.dummy import Pool as ThreadPool
import subprocess
import urllib2
import json
import socket
import traceback
import pickle
import requests
import os.path

class Task(object):
    def __init__(self, config):
        self.config = config
        self.tasks = [];
        self.unfinished_tasks = [];

    def get_updates(self):
        """get update and save to self.tasks"""
        if self.config.server is None:
            print 'config error'
            exit(0)

        request_tasks_url = 'http://' + self.config.server + '/api/tasks/new'
        print 'start request'
        start = time()
        req = False
        try:
            req = urllib2.urlopen(request_tasks_url, timeout = 3)
            response = json.load(req)
            end = time()
            print 'duration: ', int(end - start)
            self.tasks = []
            if os.path.isfile(self.config.cache_file):
                try:
                    self.unfinished_tasks = self.load_obj(self.config.cache_file)
                except:
                    print 'load object from file error'
            self.tasks.extend(self.unfinished_tasks)
            self.tasks.extend(response)
            self.unfinished_tasks = []

        except Exception as inst:
            print '----FAIL----'
            print type(inst)
            print inst.args
            print inst
            traceback.print_exc()

    def handle_tasks(self):
        pool = ThreadPool(10)
        pool.map(self.run_command, self.tasks)
        pool.close()
        pool.join()

        return True

    def run_command(self, task):
        # TODO: handle task run fail
        """run command"""
        file = task.get('file', None)
        finished = task.get('finished', None)
        if file:
            (output, err) = subprocess.Popen(task['file'], stdout=subprocess.PIPE, shell=True).communicate()
            task['result'] = output
        return True

    def send_back_result(self):
        """send back to server immediatelly"""
        print 'we have ' + str(len(self.tasks)) + ' tasks'
        while(len(self.tasks) > 0):
            task = self.tasks.pop()
            data = {"_id": task['_id'], 'result': task['result']}
            headers = {'Content-type': 'application/json'}
            url = "http://" + self.config.server + "/api/tasks/" + task['_id']
            response = requests.patch(url, data=json.dumps(data), headers=headers)
            print response.status_code
            if(response.status_code is 200):
                print 'remove task'
            else:
                self.unfinished_tasks.append(task)
        self.save_obj(self.unfinished_tasks, self.config.cache_file)
        print 'sent_back_result'
        print len(self.tasks)
        return True

    def get_interval(self):
        return 10
        return self.config.interval

    def set_interval(self, interval):
        self.config.interval = int(interval)

    def sleep(self):
        sleep(self.config.interval)

    def send_again(self):
        print 'send_again'
        return True

    def save_obj(self, obj, file_path):
        with open(file_path, 'wb') as f:
            pickle.dump(obj, f, pickle.HIGHEST_PROTOCOL)

    def load_obj(self, file_path):
        with open(file_path, 'rb') as f:
            return pickle.load(f)

    def post_data_to_server(self, url, data):
        return True



# -*- coding: utf-8 -*-

from setuptools import setup

setup(name='Empire-Agent',
      version='0.0.3',
      description='Empire Agent',
      url='http://github.com/wenxer/empire-agent',
      author='Shown Tien',
      author_email='hightian@gmail.com',
      license='MIT',
      packages=['empire_agent'],
      zip_safe=False)
